use crate::handle_messages::handle_other_message;
use crate::heartbeat::handle_heartbeat;
use crate::json_utils::{make_common_resp, parse_message};
use crate::utils;
use futures::{SinkExt, StreamExt};
use std::collections::HashMap;
use std::time::{Duration, Instant};
use tokio::time;
use tokio_tungstenite::accept_hdr_async;
use tungstenite::handshake::server::{Request, Response};
use tungstenite::{Error, Message};

// 提取出来的处理握手的函数
fn handle_handshake(
    req: &Request,
    must_existed_params: &[&str],
) -> Result<HashMap<String, String>, String> {
    println!("新客户端连接: {}", req.uri());
    let connection_params = match utils::get_connection_params(req.uri().to_string()) {
        Ok(p) => p,
        Err(e) => {
            let error_msg = format!("缺少重要连接数据段: {}", e);
            println!("{}", error_msg);
            return Err(error_msg);
        }
    };

    let not_existed = must_existed_params
        .iter()
        .find(|param| !connection_params.contains_key(&param.to_string()));

    if let Some(param) = not_existed {
        let error_msg = format!("缺少重要连接数据段: {}", param);
        println!("{}", error_msg);
        return Err(error_msg);
    }

    Ok(connection_params)
}

pub(crate) async fn handle_client(stream: tokio::net::TcpStream) -> Result<(), Error> {
    let must_existed_params = ["deviceId", "fromId", "wsPwd"];
    let mut connection_params = None;

    let ws_stream = match accept_hdr_async(stream, |req: &Request, resp| {
        match handle_handshake(req, &must_existed_params) {
            Ok(params) => {
                connection_params = Some(params);
                Ok(resp)
            }
            Err(error_msg) => {
                // 可以在这里根据需要进一步处理错误响应，这里简单打印错误信息
                println!("{}", error_msg);
                let error_resp = Response::builder()
                    .status(400)
                    .header("Content-Type", "text/plain")
                    .body(Some(error_msg))
                    .unwrap();
                Err(error_resp)
            }
        }
    })
    .await
    {
        Ok(ws) => ws,
        Err(e) => {
            println!("WebSocket握手失败: {}", e);
            return Ok(());
        }
    };

    // 将 WebSocketStream 拆分为发送器和接收器
    let (mut sender, mut receiver) = ws_stream.split();

    if let Some(params) = connection_params {
        if let Some(from_id) = params.get("fromId") {
            let from_id = from_id.clone();
            let mut last_heartbeat_time = Instant::now();

            loop {
                tokio::select! {
                    // 处理消息接收
                    maybe_msg = receiver.next() => {
                        match maybe_msg {
                            Some(Ok(msg)) => {
                                if msg.is_text() {
                                    let text = msg.to_text()?;
                                    match parse_message(text) {
                                        Ok(data) => {
                                            match data.msg_type.as_str() {
                                                "Heart" => {
                                                    println!("收到客户端心跳消息 {:?}", &data);
                                                    handle_heartbeat(&mut last_heartbeat_time);
                                                    if let Ok(json_str) = make_common_resp(Default::default(), "Heart") {
                                                        sender.send(Message::text(json_str)).await?;
                                                    }
                                                },
                                                _ => {
                                                    handle_other_message(&mut sender, &data).await?;
                                                }
                                            }
                                        }
                                        Err(e) => {
                                            println!("解析JSON数据出错: {}", e);
                                        }
                                    }
                                }
                            }
                            Some(Err(e)) => {
                                println!("接受客户端消息出错: {}", e);
                                break;
                            }
                            None => {
                                println!("客户端断开连接");
                                break;
                            }
                        }
                    }
                    // 处理心跳超时
                    _ = time::sleep_until(tokio::time::Instant::from(last_heartbeat_time + Duration::from_secs(20))) => {
                        println!("用户id-{} 20秒内没有发送心跳，挂断连接", from_id);
                        break;
                    }
                }
            }

            println!("断开与用户id: {}，连接", from_id);
        }
    } else {
        println!("无法获取连接参数");
    }

    Ok(())
}
